import { defineStore } from "pinia";
import { Store } from "@/types/student-client";
import { messageListApi } from "@/api/siteMessage";

const initStore: Store = {
  user: {} as Store["user"],
  menus: {} as Store["menus"],
  appDownload: {} as Store["appDownload"],
  exam: {} as Store["exam"],
  endedExamList: {} as Store["endedExamList"],
  examList: {} as Store["examList"],
  homeworkList: {} as Store["homeworkList"],
  practiceList: {} as Store["practiceList"],
  siteMessage: {} as Store["siteMessage"],
  siteMessagesTimeStamp: Date.now(),
  globalMaskCount: 0,
  QECSConfig: {} as Store["QECSConfig"],
  network: {} as Store["network"],
  sysTime: {} as Store["sysTime"],
} as Store;

const resetState: Omit<
  Store,
  "globalMaskCount" | "network" | "sysTime" | "QECSConfig" | "camera"
> = {
  user: {} as Store["user"],
  menus: {} as Store["menus"],
  appDownload: {} as Store["appDownload"],
  exam: {} as Store["exam"],
  endedExamList: {} as Store["endedExamList"],
  examList: {} as Store["examList"],
  homeworkList: {} as Store["homeworkList"],
  practiceList: {} as Store["practiceList"],
  siteMessage: {} as Store["siteMessage"],
  siteMessagesTimeStamp: Date.now(),
};

export function resetStore() {
  Object.assign(store.$state, resetState);
}

export const useStore = defineStore("ecs", {
  state: () => {
    return initStore;
  },
  getters: {
    /** 当前是否有globalMask */
    hasGlobalMask(): boolean {
      return store.globalMaskCount > 0;
    },
  },
  actions: {
    /** 增加当前的globalMaskCount */
    increaseGlobalMaskCount(byWho: string) {
      store.globalMaskCount++;
      console.debug(byWho, "increaseGlobalMaskCount", store.globalMaskCount);
    },
    /** 减少当前的globalMaskCount */
    decreaseGlobalMaskCount(byWho: string) {
      store.globalMaskCount--;
      console.debug(byWho, "decreaseGlobalMaskCount", store.globalMaskCount);
      if (store.globalMaskCount < 0) {
        console.error("store.globalMaskCount < 0");
        store.globalMaskCount = 0;
      }
    },
    updateTimeDifference(timeDiff: number) {
      store.sysTime.difference = timeDiff;
    },
    updateSiteMessagesTimeStamp() {
      store.siteMessagesTimeStamp = Date.now();
    },
    async fetchSiteMessage() {
      const res = await messageListApi(
        store.user.id,
        store.siteMessagesTimeStamp
      );
      const messages = res.data.map((item) => {
        const nitem = { ...item };
        if (item.hasRecalled) {
          nitem.title = "发送者已撤回消息：" + nitem.title;
          nitem.content = "该消息已经被发送者撤回。";
        }
        return nitem;
      });
      const prevSiteMessage = Object.assign(
        {},
        {
          messages: [],
          firstUnreadMessage: null,
          ignoreMessageIds: [],
          unreadCount: 0,
        },
        store.siteMessage
      );
      store.siteMessage = Object.assign({}, prevSiteMessage, { messages });
    },
  },
});

export let store = null as unknown as ReturnType<typeof useStore>;

export function setStore(s: ReturnType<typeof useStore>) {
  store = s;
}
void Promise.resolve(0).then(() => {
  // store = useStore();

  store.$subscribe((_mutation, state) => {
    sessionStorage.setItem(
      "ecs",
      JSON.stringify(
        Object.assign(JSON.parse(JSON.stringify(state)), { globalMaskCount: 0 })
      )
    );
  });
  const cachedState = JSON.parse(sessionStorage.getItem("ecs") || "0");
  cachedState && (store.$state = cachedState);
});
